@@ -278,7 +278,6 @@ swf_object_get_pngdata(swf_object_t *swf, unsigned long *length, int image_id) {
     return data;
 }
 
-
 int
 swf_object_replace_pngdata(swf_object_t *swf, int image_id,
                             unsigned char *png_data,
@@ -299,6 +298,48 @@ swf_object_replace_pngdata(swf_object_t *swf, int image_id,
     return result;
 }
 
+unsigned char *
+swf_object_get_sounddata(swf_object_t *swf, unsigned long *length, int sound_id) {
+    swf_tag_t *tag;
+    unsigned char *data = NULL;
+    *length = 0;
+    if (swf == NULL) {
+        fprintf(stderr, "swf_object_get_sounddata: swf == NULL\n");
+        return NULL;
+    }
+    for (tag = swf->tag; tag; tag = tag->next) {
+        // DefineSound
+        if (tag->tag != 14) {
+            continue;
+        }
+        data = swf_tag_get_sound_data(tag, length, sound_id);
+        if (data) {
+            break;
+        }
+    }
+    return data;
+}
+
+int
+swf_object_replace_melodata(swf_object_t *swf, int sound_id,
+                            unsigned char *melo_data,
+                            unsigned long melo_data_len) {
+    int result = 1;
+    swf_tag_t *tag;
+    if (swf == NULL) {
+        fprintf(stderr, "swf_object_replace_melodata: swf == NULL\n");
+        return 1;
+    }
+    for (tag = swf->tag; tag; tag = tag->next) {
+        result = swf_tag_replace_melo_data(tag, sound_id,
+                                           melo_data, melo_data_len);
+        if (! result) {
+            break;
+        }
+    }
+    return result;
+}
+
 char *
 swf_object_get_editstring(swf_object_t *swf,
                           char *variable_name,
