@@ -66,8 +66,7 @@ swf_tag_action_output_detail(void *detail, unsigned long *length,
                              struct swf_object_ *swf) {
     swf_tag_action_detail_t *swf_tag_action = (swf_tag_action_detail_t *) detail;
     bitstream_t *bs;
-    unsigned char *data, *d;
-    unsigned long len;
+    unsigned char *data;
     *length = 0;
 
     (void) swf;
