@@ -29,12 +29,12 @@
 
 #include "swf_define.h"
 #include "swf_tag_jpeg.h"
+#include "swf_tag_lossless.h"
 #include "swf_tag_edit.h"
+#include "swf_tag_sound.h"
 #include "swf_tag_action.h"
 #include "swf_tag.h"
 #include "swf_object.h"
-#include "swf_tag_action.h"
-// #include "swf_action.h"
 
 /* If you declare any globals in php_swfed.h uncomment this:
 ZEND_DECLARE_MODULE_GLOBALS(swfed)
@@ -62,6 +62,8 @@ zend_function_entry swfed_functions[] = {
    	PHP_ME(swfed,  replaceJpegData, NULL, 0)
         PHP_ME(swfed,  getPNGData, NULL, 0)
         PHP_ME(swfed,  replacePNGData, NULL, 0)
+        PHP_ME(swfed,  getSoundData, NULL, 0)
+        PHP_ME(swfed,  replaceMLDData, NULL, 0)
    	PHP_ME(swfed,  getEditString, NULL, 0)
    	PHP_ME(swfed,  replaceEditString, NULL, 0)
    	PHP_ME(swfed,  getActionData, NULL, 0)
@@ -360,9 +362,11 @@ PHP_METHOD(swfed, getTagDetail) {
     }
     swf_tag_create_detail(tag, swf);
     switch (tag->tag) {
-        swf_tag_jpeg_detail_t   *tag_jpeg;
-        swf_tag_edit_detail_t   *tag_edit;
-        swf_tag_action_detail_t *tag_action;
+        swf_tag_jpeg_detail_t     *tag_jpeg;
+        swf_tag_lossless_detail_t *tag_lossless;
+        swf_tag_edit_detail_t     *tag_edit;
+        swf_tag_sound_detail_t    *tag_sound;
+        swf_tag_action_detail_t   *tag_action;
       case 6:  // DefineBitsJPEG
       case 21: // DefineBitsJPEG2
       case 35: // DefineBitsJPEG3
@@ -375,6 +379,29 @@ PHP_METHOD(swfed, getTagDetail) {
             add_assoc_long(return_value, "alpha_data_len", tag_jpeg->alpha_data_len);
         }
         break;
+      case 20: // DefineBitsLossless
+      case 36: // DefineBitsLossless2
+        tag_lossless = tag->detail;
+        array_init(return_value);
+        add_assoc_long(return_value, "image_id", tag_lossless->image_id);
+        add_assoc_long(return_value, "format", tag_lossless->format);
+        add_assoc_long(return_value, "width", tag_lossless->width);
+        add_assoc_long(return_value, "height", tag_lossless->height);
+        if (tag_lossless->format == 3) {
+            add_assoc_long(return_value, "colormap_count", tag_lossless->colormap_count);
+        }
+        break;
+    case 14: // DefineSound
+        tag_sound = tag->detail;
+        array_init(return_value);
+        add_assoc_long(return_value, "sound_id", tag_sound->sound_id);
+        add_assoc_long(return_value, "format", (unsigned long) tag_sound->sound_format);
+        add_assoc_long(return_value, "rate", (unsigned long) tag_sound->sound_rate);
+        add_assoc_long(return_value, "is_16bits", tag_sound->sound_is_16bits?1:0);
+        add_assoc_long(return_value, "is_stereo", tag_sound->sound_is_stereo?1:0);
+        add_assoc_long(return_value, "sound_samples_count", tag_sound->sound_samples_count);
+        add_assoc_long(return_value, "sound_data_len", tag_sound->sound_data_len);
+        break;
     case 12: // DoAction
     case 59: // DoInitAction
         tag_action = tag->detail;
@@ -544,6 +571,58 @@ PHP_METHOD(swfed, replacePNGData) {
     RETURN_TRUE;
 }
 
+PHP_METHOD(swfed, getSoundData) {
+    unsigned long sound_id = 0;
+    unsigned long len = 0;
+    unsigned char *data, *new_buff;
+    zval *obj = getThis();
+    swf_object_t *swf;
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
+                              "l", &sound_id) == FAILURE) {
+        RETURN_FALSE;
+    }
+    swf = get_swf_object(getThis());
+    data = swf_object_get_sounddata(swf, &len, sound_id);
+    if (data == NULL) {
+        RETURN_FALSE;
+    }
+    new_buff = emalloc(len);
+    if (new_buff == NULL) {
+        fprintf(stderr, "getSoundData: Can't emalloc new_buff\n");
+        free(data);
+        RETURN_FALSE;
+    }
+    memcpy(new_buff, data, len);
+    free(data);
+    RETURN_STRINGL(new_buff, (int) len, 1);
+}
+
+PHP_METHOD(swfed, replaceMLDData) {
+    char *data = NULL;
+    int data_len = 0;
+    int sound_id;
+    swf_object_t *swf;
+    int result = 0;
+    switch (ZEND_NUM_ARGS()) {
+      default:
+        WRONG_PARAM_COUNT;
+        RETURN_FALSE; /* XXX */
+      case 2:
+        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ls", &sound_id, &data, &data_len) == FAILURE) {
+            RETURN_FALSE;
+        }
+        break;
+    }
+    swf = get_swf_object(getThis());
+    result = swf_object_replace_melodata(swf, sound_id,
+                                         (unsigned char *)data,
+                                         (unsigned long) data_len);
+    if (result == 0) {
+        RETURN_FALSE;
+    }
+    RETURN_TRUE;
+}
+
 PHP_METHOD(swfed, getEditString) {
     char *var_name;
     int  var_name_len;
